@@ -36,11 +36,12 @@ from django.utils._decimal import Decimal
 from django.core.serializers import register_serializer
 from django.contrib.contenttypes.models import ContentType
 
-from restclient import RestClient
+from restkit import RestClient
 from django_roa.remoteauth.models import User, Message, Group, Permission
 from django_roa_client.models import RemotePage, RemotePageWithManyFields, \
     RemotePageWithBooleanFields, RemotePageWithRelations, \
-    RemotePageWithCustomSlug, RemotePageWithOverriddenUrls
+    RemotePageWithCustomSlug, RemotePageWithOverriddenUrls, \
+    RemotePageWithNamedRelations, RemotePageWithProxy
 from django_roa_client.forms import TestForm, RemotePageForm
 
 class ROATestCase(TestCase):
@@ -110,11 +111,17 @@ class ROABaseTests(ROATestCase):
 class ROAUnicodeTests(ROATestCase):
     
     def test_remotepage(self):
-        emilie = RemotePage.objects.create(title=u"\xc9milie")
+        emilie = RemotePage.objects.create(title=u"Émilie")
         self.assertEqual(emilie.title, u'Émilie')
-        emilie = RemotePage.objects.get(title=u"\xc9milie")
+        emilie = RemotePage.objects.get(title=u"Émilie")
         self.assertEqual(emilie.title, u'Émilie')
-        emilie.delete()
+        amelie = emilie
+        amelie.title = u'Amélie'
+        amelie.save()
+        self.assertEqual(amelie.title, u'Amélie')
+        amelie = RemotePage.objects.get(title=u"Amélie")
+        self.assertEqual(amelie.title, u'Amélie')
+        amelie.delete()
 
 
 class ROAFieldsTests(ROATestCase):
@@ -341,6 +348,7 @@ class ROARelationsTests(ROATestCase):
         super(ROARelationsTests, self).tearDown()
         RemotePageWithManyFields.objects.all().delete()
         RemotePageWithRelations.objects.all().delete()
+        RemotePageWithNamedRelations.objects.all().delete()
 
     def test_empty_relation(self):
         relations_page = RemotePageWithRelations.objects.create()
@@ -360,6 +368,7 @@ class ROARelationsTests(ROATestCase):
         self.assertEqual(repr(relations_page.remote_page), '<RemotePage: A remote page (1)>')
         relations_page = RemotePageWithRelations.objects.get(id=relations_page.id)
         self.assertEqual(repr(relations_page.remote_page), '<RemotePage: A remote page (1)>')
+        self.assertEqual(repr(remote_page.remotepagewithrelations_set.all()), '[<RemotePageWithRelations:  (1)>]')
         relations_page.remote_page = another_remote_page
         relations_page.save()
         relations_page = RemotePageWithRelations.objects.get(id=relations_page.id)
@@ -388,6 +397,39 @@ class ROARelationsTests(ROATestCase):
         another_remote_page.delete()
         remote_page.delete()
         
+    def test_named_relation(self):
+        remote_page = RemotePage.objects.create(title=u'A remote page')
+        another_remote_page = RemotePage.objects.create(title=u'Another remote page')
+        named_relations_page = RemotePageWithNamedRelations.objects.create(first_page=remote_page, 
+                                                                           last_page=another_remote_page)
+        self.assertEqual(repr(named_relations_page.first_page), '<RemotePage: A remote page (1)>')
+        named_relations_page = RemotePageWithNamedRelations.objects.get(id=named_relations_page.id)
+        self.assertEqual(repr(named_relations_page.first_page), '<RemotePage: A remote page (1)>')
+        self.assertEqual(repr(remote_page.from_first.all()), '[<RemotePageWithNamedRelations:  (1)>]')
+        self.assertEqual(repr(another_remote_page.from_last.all()), '[<RemotePageWithNamedRelations:  (1)>]')
+        named_relations_page.first_page = another_remote_page
+        named_relations_page.save()
+        named_relations_page = RemotePageWithNamedRelations.objects.get(id=named_relations_page.id)
+        self.assertEqual(repr(named_relations_page.first_page), '<RemotePage: Another remote page (2)>')
+        named_relations_page.delete()
+        another_remote_page.delete()
+        remote_page.delete()
+
+    def test_proxy_relation(self):
+        remote_page = RemotePage.objects.create(title=u'A remote page')
+        proxy_remote_page = RemotePageWithProxy.objects.create(title=u'A proxy remote page')
+        self.assertEqual(repr(remote_page), '<RemotePage: A remote page (1)>')
+        self.assertEqual(repr(proxy_remote_page), '<RemotePageWithProxy: A proxy remote page (2)>')
+        self.assertEqual(repr(RemotePage.objects.all()), '[<RemotePage: A remote page (1)>, <RemotePage: A proxy remote page (2)>]')
+        self.assertEqual(repr(RemotePageWithProxy.objects.all()), '[<RemotePage: A remote page (1)>, <RemotePage: A proxy remote page (2)>]')
+        proxy_remote_page.title = u'A modified proxy remote page'
+        proxy_remote_page.save()
+        proxy_remote_page = RemotePageWithProxy.objects.get(id=2)
+        self.assertEqual(repr(proxy_remote_page), '<RemotePage: A modified proxy remote page (2)>')
+        self.assertEqual(repr(RemotePage.objects.all()[1]), '<RemotePage: A modified proxy remote page (2)>')
+        proxy_remote_page.delete()
+        remote_page.delete()
+
 
 class ROAQuerysetTests(ROATestCase):
     
@@ -420,6 +462,11 @@ class ROAQuerysetTests(ROATestCase):
         self.assertEqual(repr(RemotePage.objects.all()[1:3]), '[<RemotePage: Another remote page (2)>, <RemotePage: Yet another remote page (3)>]')
         self.assertEqual(repr(RemotePage.objects.all()[0]), '<RemotePage: A remote page (1)>')
         
+    def test_extra(self):
+        self.assertEqual(bool(RemotePage.objects.all().extra(select={'a': 1}).values('a').order_by()), True)
+        RemotePage.objects.all().delete()
+        self.assertEqual(bool(RemotePage.objects.all().extra(select={'a': 1}).values('a').order_by()), False)
+        
     def test_combined(self):
         self.assertEqual(repr(RemotePage.objects.exclude(title__contains='yet').order_by('title', '-id')[:2]), '[<RemotePage: A remote page (1)>, <RemotePage: Another remote page (2)>]')
 
